/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package net.rmi.rjs.pk.LusCs;

import gui.*;
import gui.layouts.DialogLayout;
import gui.run.RunCheckBox;

import javax.swing.*;
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
import java.util.Date;

/**
 * Copyright DocJava, inc. User: lyon
 * <p/>
 * Date: May 1, 2005
 * <p/>
 * Time: 5:54:44 PM
 */
public class CsRecordPanel extends JPanel implements Observer {
    private CsRecord csr;
    private JLabel speedLabel;
    private RunCheckBox busyCheckbox;
    private JLabel ageLabel;
    private ObservablePanel observablePanel = new ObservablePanel();

    public void addObserver(Observer obs){
        observablePanel.addObserver(obs);
    }

    private void setBusy(boolean b) {
        csr.setOnTheGrid(b);
    }

    public CsRecordPanel(CsRecord csr) {
        this.csr = csr;
        speedLabel = new JLabel("      ");
        busyCheckbox = new RunCheckBox(csr.isOnTheGrid()) {
            public void run() {
                setBusy(this.isSelected());
            }


        };
        final Date age = csr.getDateOfInstance();

        ageLabel = new JLabel(age + "");
        csr.addObserver(this);
        setLayout(new DialogLayout());
        add(new JLabel("speed in flops:"));
        add(speedLabel);
        final String ipAddrString =
                csr.getIpAddress().getHostAddress();
        add(new JLabel("busy:"));
        add(busyCheckbox);
        add(new JLabel("date when instanced"));
        add(ageLabel);
        setBorder(BorderFactory.createTitledBorder(ipAddrString));
        update();
    }

    public void update() {
        speedLabel.setText(csr.getBmb().getMflops() + "");
        busyCheckbox.setSelected(csr.isOnTheGrid());
        ageLabel.setText(csr.getDateOfInstance() + "");
        if (csr.isDead()) {
            setVisible(false);
            updateFrame();
        }
        observablePanel.setChanged();
        observablePanel.notifyObservers();
    }

    private void updateFrame() {
        Container parent = getParent();
        gui.Utils.updateFrame(parent);
    }

    public void update(Observable obs, Object obj) {
        update();
    }

    public static void main(String[] args) {
        CsRecords csrs = CsRecords.getTestRecords();
        displayCsRecords(csrs);
    }

    public static void displayCsRecords(CsRecords csrs) {
        ClosableJFrame cf = new ClosableJFrame();
        JPanel jp = new JPanel();
        Container c = cf.getContentPane();
        jp.setLayout(new FlowLayout());

        CsRecord csra[] = csrs.getRecords();
        for (int i = 0; i < csra.length; i++) {
            jp.add(new CsRecordPanel(csra[i]));
        }
        jp.add(new CSRecordButtonControlPanel(csrs));
        JScrollPane jsp = new JScrollPane(jp);
        c.add(jsp);
        cf.pack();
        cf.setVisible(true);
    }

    private static class ObservablePanel extends Observable {
        public void setChanged(){
            super.setChanged();
        }
    }
}